import json
from typing import Optional

from selenium.webdriver.remote.webelement import WebElement


class ElementUtils:
    @staticmethod
    def get_offset(value: str, element: Optional[WebElement] = None) -> Optional[tuple[int, int]]:
        if not value:
            return None

        default_x = element.size['width'] if element else 0
        default_y = element.size['height'] if element else 0

        value_map = json.loads(value)
        offset_x = value_map.get('offset_x', default_x)
        offset_y = value_map.get('offset_y', default_y)
        return offset_x, offset_y


    @staticmethod
    def get_parent_offset(value: str, element: Optional[WebElement] = None) -> Optional[tuple[int, int]]:
        if not value:
            return None

        default_x = element.size['width'] if element else 0
        default_y = element.size['height'] if element else 0

        value_map = json.loads(value)
        offset_x = value_map.get('offset_X', default_x)
        offset_y = value_map.get('offset_Y', default_y)
        return offset_x, offset_y


from typing import Optional

from mag_tools.utils.common.string_utils import StringUtils

from mag_test.model.action_type import ActionType
from mag_test.model.control_type import ControlType
from mag_test.model.menu_type import MenuType
from mag_test.model.init_status import InitStatus


class Control:
    def __init__(self, name: Optional[str] = None, control_type: Optional[ControlType] = None, automation_id: Optional[str] = None,
                 action: Optional[ActionType] = None, class_name: Optional[str] = None,
                 menu_type: Optional[MenuType] = None, init_status: Optional[InitStatus] = None):
        self.name = name
        self.control_type = control_type
        self.automation_id = automation_id
        self.action = action
        self.menu_type = menu_type
        self.class_name = class_name
        self.init_status = init_status

    def get_from_name_path(self, name_path: str):
        if name_path:
            self.name, action_name, _ = StringUtils.split_by_keyword(name_path, '{}')
            if self.control_type in {ControlType.TREE, ControlType.TABLE}:
                self.init_status = InitStatus.of_code(action_name)
            else:
                self.action = ActionType.of_code(action_name) if action_name else ActionType.default_action(self.control_type)

    def get_from_id_path(self, id_path: str):
        if id_path:
            self.automation_id, action_name, _ = StringUtils.split_by_keyword(id_path, '{}')
            if self.control_type in {ControlType.TREE, ControlType.TABLE}:
                self.init_status = InitStatus.of_code(action_name)
            else:
                self.action = ActionType.of_code(action_name) if action_name else ActionType.default_action(self.control_type)

    @property
    def is_composite(self):
        return self.control_type.is_composite

    def __str__(self) -> str:
        attributes = {k: v for k, v in self.__dict__.items() if v is not None}
        return f"Control({', '.join(f'{k}={v}' for k, v in attributes.items())})"



from typing import Optional

from appium import webdriver

from selenium.common.exceptions import NoSuchElementException
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.wait import WebDriverWait

from mag_test.model.control_type import ControlType
from mag_test.finder.element_finder_utils import ElementFinderUtils


class DriverFinderUtils:
    @staticmethod
    def find_element_by_type(driver: webdriver.Remote, name: Optional[str], control_type: Optional[ControlType] = None, parent_name: Optional[str] = None, parent_control_type: Optional[ControlType] = None):
        if name:
            exp = ElementFinderUtils.global_expression(name, control_type, None, None, parent_name, parent_control_type)
            return driver.find_element(By.XPATH, exp)
        else:
            elements = DriverFinderUtils.find_elements_by_type(driver, None, control_type, parent_name, parent_control_type)
            for element in elements:
                if element.get_attribute("Name") is None:
                    return element

    @staticmethod
    def find_element_by_class(driver: webdriver.Remote, name: Optional[str], class_name: Optional[str] = None, parent_name: Optional[str] = None, parent_control_type: Optional[ControlType] = None):
        if name:
            exp = ElementFinderUtils.global_expression(name, None, class_name, None, parent_name, parent_control_type)
            return driver.find_element(By.XPATH, exp)
        else:
            elements = DriverFinderUtils.find_elements_by_class(driver, None, class_name, parent_name, parent_control_type)
            for element in elements:
                if element.get_attribute("Name") is None:
                    return element

    @staticmethod
    def find_element_by_automation(driver: webdriver.Remote, automation_id: Optional[str]):
        exp = ElementFinderUtils.global_expression(None, None, None, automation_id, None, None)
        return driver.find_element(By.XPATH, exp)

    @staticmethod
    def find_element_by_type_wait(driver: webdriver.Remote, name: Optional[str], control_type: Optional[ControlType] = None, parent_name: Optional[str] = None, parent_control_type: Optional[ControlType] = None):
        if name:
            exp = ElementFinderUtils.global_expression(name, control_type, None, None, parent_name, parent_control_type)
            wait = WebDriverWait(driver, 5)
            return wait.until(EC.presence_of_element_located((By.XPATH, exp)))
        else:
            elements = DriverFinderUtils.find_elements_by_type_wait(driver, None, control_type, parent_name, parent_control_type)
            for element in elements:
                if element.get_attribute("Name") is None:
                    return element

    @staticmethod
    def find_element_by_class_wait(driver: webdriver.Remote, name: Optional[str], class_name: Optional[str] = None, parent_name: Optional[str] = None, parent_control_type: Optional[ControlType] = None):
        if name:
            exp = ElementFinderUtils.global_expression(name, None, class_name, None, parent_name, parent_control_type)
            wait = WebDriverWait(driver, 5)
            return wait.until(EC.presence_of_element_located((By.XPATH, exp)))
        else:
            elements = DriverFinderUtils.find_elements_by_class_wait(driver, None, class_name, parent_name, parent_control_type)
            for element in elements:
                if element.get_attribute("Name") is None:
                    return element

    @staticmethod
    def find_element_by_automation_wait(driver: webdriver.Remote, automation_id: Optional[str]):
        exp = ElementFinderUtils.global_expression(None, None, None, automation_id, None, None)
        wait = WebDriverWait(driver, 5)
        return wait.until(EC.presence_of_element_located((By.XPATH, exp)))

    @staticmethod
    def find_elements_by_type_wait(driver: webdriver.Remote, name: Optional[str], control_type: Optional[ControlType] = None, parent_name: Optional[str] = None, parent_control_type: Optional[ControlType] = None):
        exp = ElementFinderUtils.global_expression(name, control_type, None, None, parent_name, parent_control_type)
        wait = WebDriverWait(driver, 5)
        return wait.until(EC.presence_of_all_elements_located((By.XPATH, exp)))

    @staticmethod
    def find_elements_by_class_wait(driver: webdriver.Remote, name: Optional[str], class_name: Optional[str] = None, parent_name: Optional[str] = None, parent_control_type: Optional[ControlType] = None):
        exp = ElementFinderUtils.global_expression(name, None, class_name, None, parent_name, parent_control_type)
        wait = WebDriverWait(driver, 5)
        return wait.until(EC.presence_of_all_elements_located((By.XPATH, exp)))

    @staticmethod
    def find_elements_by_type(driver: webdriver.Remote, name: Optional[str], control_type: Optional[ControlType] = None, parent_name: Optional[str] = None, parent_control_type: Optional[ControlType] = None):
        exp = ElementFinderUtils.global_expression(name, control_type, None, None, parent_name, parent_control_type)
        return driver.find_elements(By.XPATH, exp)

    @staticmethod
    def find_elements_by_class(driver: webdriver.Remote, name: Optional[str], class_name: Optional[str] = None, parent_name: Optional[str] = None, parent_control_type: Optional[ControlType] = None):
        exp = ElementFinderUtils.global_expression(name, None, class_name, None, parent_name, parent_control_type)
        return driver.find_elements(By.XPATH, exp)

    @staticmethod
    def find_element_by_types(driver: webdriver.Remote, name: Optional[str], control_types: list[ControlType] = None, parent_name: Optional[str] = None, parent_control_type: Optional[ControlType] = None):
        element = None
        for control_type in control_types:
            try:
                element = DriverFinderUtils.find_element_by_type(driver, name, control_type, parent_name, parent_control_type)
                break
            except NoSuchElementException:
                pass
        return element